@@ -9,8 +9,8 @@ import numpy as np
 from astropy import units as u
 
 
-__all__ = ['Base', 'BaseTaskBase', 'SetAttribute', 'TaskBase',
-           'Task', 'PaddedTaskBase']
+__all__ = ['Base', 'BaseTaskBase', 'TaskBase', 'PaddedTaskBase',
+           'SetAttribute', 'Task']
 
 
 def check_broadcast_to(value, sample_shape):
@@ -96,7 +96,6 @@ class Base:
         self._shape = shape
         self._start_time = start_time
         self._samples_per_frame = operator.index(samples_per_frame)
-        assert shape[0] % samples_per_frame == 0
         self._sample_rate = sample_rate
         self._dtype = np.dtype(dtype, copy=False)
         if frequency is not None or sideband is not None:
@@ -114,6 +113,63 @@ class Base:
         self._sideband = sideband
         self._polarization = polarization
 
+    def _repr_item(self, key, default, value=None):
+        """Representation of one argument.
+
+        Subclasses can override this, either to return something else than
+        the base key=value or to set a different default for specific keys.
+
+        """
+
+        if value is None:
+            value = getattr(self, key, None)
+            if value is None:
+                value = getattr(self, '_' + key, None)
+                if value is None:
+                    return None
+
+        if default is not inspect._empty:
+            try:
+                if np.all(value == default):
+                    return None
+            except Exception:
+                pass
+
+        return f"{key}={value}".replace('\n', ',')
+
+    def __str__(self):
+        name = self.__class__.__name__
+        pars = inspect.signature(self.__class__).parameters
+        overrides = [self._repr_item(key, par.default)
+                     for key, par in pars.items()]
+
+        overrides = ', '.join([override for override in overrides if override])
+        return f"{name}({overrides})"
+
+    def __repr__(self):
+        """Representation which lists non-default arguments.
+
+        Finds possible arguments by inspection of the whole class hierarchy
+        (as long as kwargs are passed along) and creates a list of all whose
+        values on the instance are different from the default. Subclasses
+        can override the assumed default and what to return in _repr_item.
+
+        """
+        name = self.__class__.__name__
+        pars = {}
+        for cls in self.__class__.__mro__:
+            for key, par in inspect.signature(cls).parameters.items():
+                pars.setdefault(key, par)
+            if 'kwargs' not in pars:
+                break
+
+        overrides = [self._repr_item(key, par.default)
+                     for key, par in pars.items()]
+
+        overrides = (',\n '+' '*len(name)).join(
+            [override for override in overrides if override])
+        return f"{name}({overrides})"
+
     def _check_shape(self, value):
         """Check that value can be broadcast to the sample shape."""
         broadcast = check_broadcast_to(value, self.sample_shape)
@@ -171,7 +227,9 @@ class Base:
 
         See also `time` and `stop_time`.
         """
-        return self._start_time
+        # We don't just return self._start_time so classes like Integrate
+        # can get correct results by just overriding _tell_time.
+        return self._tell_time(0)
 
     @property
     def time(self):
@@ -179,7 +237,7 @@ class Base:
 
         See also `start_time` and `stop_time`.
         """
-        return self.tell(unit='time')
+        return self._tell_time(self.offset)
 
     @property
     def stop_time(self):
@@ -187,7 +245,7 @@ class Base:
 
         See also `start_time` and `time`.
         """
-        return self.start_time + self.shape[0] / self.sample_rate
+        return self._tell_time(self.shape[0])
 
     @property
     def frequency(self):
@@ -210,6 +268,9 @@ class Base:
     def seek(self, offset, whence=0):
         """Change the sample pointer position.
 
+        This works like a normal filehandle seek, but the offset is in samples
+        (or a relative or absolute time).
+
         Parameters
         ----------
         offset : int, `~astropy.units.Quantity`, or `~astropy.time.Time`
@@ -267,47 +328,53 @@ class Base:
 
         # "isinstance" avoids costly comparisons of an actual unit with 'time'.
         if not isinstance(unit, u.UnitBase) and unit == 'time':
-            return self.start_time + self.tell(unit=u.s)
+            return self._tell_time(self.offset)
 
         return (self.offset / self.sample_rate).to(unit)
 
+    def _tell_time(self, offset):
+        """Calculate time for given offset.
+
+        Used for ``start_time``, ``time``, ``stop_time`` and
+        ``tell(unit='time')``.  Simple implementation is present mostly so
+        subclasses like Integration and Stack can override as appropriate.
+
+        """
+        return self._start_time + offset / self.sample_rate
+
     def read(self, count=None, out=None):
         """Read a number of complete samples.
 
         Parameters
         ----------
         count : int or None, optional
-            Number of complete samples to read.  If `None` (default) or
-            negative, the entire input data is processed.  Ignored if ``out``
-            is given.
+            Number of complete samples to read. If `None` (default) or
+            negative, the number of samples left. Ignored if ``out`` is given.
         out : None or array, optional
-            Array to store the output in. If given, ``count`` will be inferred
-            from the first dimension; the other dimension should equal
+            Array to store the samples in. If given, ``count`` will be inferred
+            from the first dimension; the remaining dimensions should equal
             `sample_shape`.
 
         Returns
         -------
         out : `~numpy.ndarray` of float or complex
-            The first dimension is sample-time, and the remainder given by
-            `sample_shape`.
+            The first dimension is sample-time, and the remaining ones are
+            as given by `sample_shape`.
         """
         if self.closed:
-            raise ValueError("I/O operation on closed task/generator.")
+            raise ValueError("I/O operation on closed stream.")
 
-        # NOTE: this will return an EOF error when attempting to read partial
-        # frames, making it identical to fh.read().
-
-        samples_left = max(0, self.shape[0] - self.offset)
+        samples_left = self.shape[0] - self.offset
         if out is None:
             if count is None or count < 0:
-                count = samples_left
-            out = np.empty((count,) + self.shape[1:], dtype=self.dtype)
+                count = max(0, samples_left)
+
+            out = np.empty((count,) + self.sample_shape, dtype=self.dtype)
         else:
-            assert out.shape[1:] == self.shape[1:], (
+            assert out.shape[1:] == self.sample_shape, (
                 "'out' must have trailing shape {}".format(self.sample_shape))
             count = out.shape[0]
 
-        # TODO: should this just return the maximum possible?
         if count > samples_left:
             raise EOFError("cannot read from beyond end of input.")
 
@@ -315,25 +382,65 @@ class Base:
         sample = 0
         while sample < count:
             # For current position, get frame plus offset in that frame.
-            frame_index, sample_offset = divmod(self.offset,
-                                                self._samples_per_frame)
-
-            if frame_index != self._frame_index:
-                # Read the frame required.  Set offset at the start so
-                # that _read_frame can count on tell() being correct.
-                self.offset = frame_index * self._samples_per_frame
-                self._frame = self._read_frame(frame_index)
-                self._frame_index = frame_index
-
-            nsample = min(count - sample, len(self._frame) - sample_offset)
-            data = self._frame[sample_offset:sample_offset + nsample]
+            frame, sample_offset = self._get_frame(self.offset)
+            nsample = min(count - sample, len(frame) - sample_offset)
+            data = frame[sample_offset:sample_offset + nsample]
             # Copy to relevant part of output.
             out[sample:sample + nsample] = data
             sample += nsample
+            # Explicitly set offset (leaving get_frame free to adjust it).
             self.offset = offset0 + sample
 
         return out
 
+    def _get_frame(self, offset):
+        """Get a frame that includes given offset.
+
+        Finds the index corresponding to the needed frame, assuming frames
+        are all the same length.  If not already cached, it retrieves a
+        frame by calling ``self._read_frame(index)``.
+
+        Parameters
+        ----------
+        offset : int
+            Offset in the stream for which a frame should be found.
+
+        Returns
+        -------
+        frame : `~baseband.base.frame.FrameBase`
+            Frame holding the sample at ``offset``.
+        sample_offset : int
+            Offset within the frame corresponding to ``offset``.
+        """
+        frame_index, sample_offset = divmod(offset, self.samples_per_frame)
+        if frame_index != self._frame_index:
+            # Read the frame required. Set offset to start so that _read_frame
+            # can count on tell() being correct.
+            self.offset = frame_index * self.samples_per_frame
+            self._frame = self._read_frame(frame_index)
+            self._frame_index = frame_index
+
+        return self._frame, sample_offset
+
+    def __getitem__(self, item):
+        from .shaping import GetSlice
+
+        return GetSlice(self, item)
+
+    def __array__(self, dtype=None):
+        old_offset = self.tell()
+        try:
+            self.seek(0)
+            return np.array(self.read(), dtype=dtype, copy=False)
+        finally:
+            self.seek(old_offset)
+
+    def __array_ufunc__(self, *args, **kwargs):
+        return NotImplemented
+
+    def __array_function__(self, *args, **kwargs):
+        return NotImplemented
+
     def __enter__(self):
         return self
 
@@ -367,6 +474,8 @@ class BaseTaskBase(Base):
     ----------
     ih : stream handle
         Handle of a stream reader or another task.
+    ih_samples_per_frame : int, optional
+        Number of input samples which should be dealt with in one go.
     shape : tuple, optional
         Overall shape of the stream, with first entry the total number
         of complete samples, and the remainder the sample shape.
@@ -375,7 +484,8 @@ class BaseTaskBase(Base):
     sample_rate : `~astropy.units.Quantity`, optional
         With units of a rate.
     samples_per_frame : int, optional
-        Number of samples to be read and processed in one go.
+        Number of output samples produced per frame.  By default, equal
+        to the number of input samples.
     frequency : `~astropy.units.Quantity`, optional
         Frequencies for each channel.  Should be broadcastable to the
         sample shape.
@@ -391,11 +501,15 @@ class BaseTaskBase(Base):
 
     """
 
-    def __init__(self, ih, *,
+    def __init__(self, ih, *, ih_samples_per_frame=None,
                  start_time=None, shape=None, sample_rate=None,
                  samples_per_frame=None, frequency=None, sideband=None,
                  polarization=None, dtype=None):
         self.ih = ih
+        if ih_samples_per_frame is None:
+            ih_samples_per_frame = ih.samples_per_frame
+        self._ih_samples_per_frame = ih_samples_per_frame
+
         if shape is None:
             shape = ih.shape
         if start_time is None:
@@ -403,7 +517,7 @@ class BaseTaskBase(Base):
         if sample_rate is None:
             sample_rate = ih.sample_rate
         if samples_per_frame is None:
-            samples_per_frame = ih.samples_per_frame
+            samples_per_frame = self._ih_samples_per_frame
         if dtype is None:
             dtype = ih.dtype
         if frequency is None:
@@ -412,10 +526,6 @@ class BaseTaskBase(Base):
             sideband = getattr(ih, 'sideband', None)
         if polarization is None:
             polarization = getattr(ih, 'polarization', None)
-        # Sanity check on shape.
-        nframes = (shape[0] // samples_per_frame) * samples_per_frame
-        assert nframes > 0, "time per frame larger than total time in stream"
-        shape = (nframes,) + shape[1:]
 
         super().__init__(shape=shape, start_time=start_time,
                          sample_rate=sample_rate,
@@ -423,6 +533,27 @@ class BaseTaskBase(Base):
                          frequency=frequency, sideband=sideband,
                          polarization=polarization, dtype=dtype)
 
+    def _repr_item(self, key, default, value=None):
+        if key == 'ih':
+            return 'ih'
+        if default is None:
+            if key == 'samples_per_frame':
+                default = self._ih_samples_per_frame
+            elif key == 'ih_samples_per_frame':
+                default = self.ih.samples_per_frame
+            else:
+                default = getattr(self.ih, key, None)
+
+        return super()._repr_item(key, default=default, value=value)
+
+    def __repr__(self):
+        base = super().__repr__()
+        if base.count('\n') == 1:
+            base = ' '.join(b.strip() for b in base.split('\n'))
+        return (base
+                + "\nih: "
+                + "\n    ".join(repr(self.ih).split('\n')))
+
     def close(self):
         """Close task.
 
@@ -435,57 +566,6 @@ class BaseTaskBase(Base):
         del self.ih
 
 
-class SetAttribute(BaseTaskBase):
-    """Wrapper for streams that allows one to set or change attributes.
-
-    Can be used to add ``frequency``, ``sideband`` and ``polarization``
-    attributes, which most baseband readers do not provide, checking that
-    the values broadcast properly to the sample shape.
-
-    Can also be used to apply a clock correction by changing ``start_time``.
-
-    The ``sample_rate`` can also be set, but no check is done to ensure it
-    remains consistent with the ``frequency``.
-
-    The class reads directly from the underlying stream, which means it has
-    very little performance impact, but also that one cannot change the
-    ``shape``, ``frames_per_sample``, or ``dtype`` of the underlying stream.
-
-    By default, all parameters are taken from the underlying stream.
-
-    Parameters
-    ----------
-    ih : stream handle
-        Handle of a stream reader or another task.
-    start_time : `~astropy.time.Time`, optional
-        Start time of the stream.
-    sample_rate : `~astropy.units.Quantity`, optional
-        With units of a rate.
-    frequency : `~astropy.units.Quantity`, optional
-        Frequencies for each channel.  Should be broadcastable to the
-        sample shape.
-    sideband : array, optional
-        Whether frequencies are upper (+1) or lower (-1) sideband.
-        Should be broadcastable to the sample shape.
-    polarization : array or (nested) list of char, optional
-        Polarization labels.  Should broadcast to the sample shape,
-        i.e., the labels are in the correct axis.  For instance,
-        ``['X', 'Y']``, or ``[['L'], ['R']]``.
-
-    """
-
-    def __init__(self, ih, *, start_time=None, sample_rate=None,
-                 frequency=None, sideband=None, polarization=None):
-        super().__init__(ih, start_time=start_time, sample_rate=sample_rate,
-                         frequency=frequency, sideband=sideband,
-                         polarization=polarization)
-
-    def read(self, *args, **kwargs):
-        """Read data from the underlying stream at the current offset."""
-        self.ih.seek(self.offset)
-        return self.ih.read(*args, **kwargs)
-
-
 class TaskBase(BaseTaskBase):
     """Base class of all tasks.
 
@@ -504,77 +584,146 @@ class TaskBase(BaseTaskBase):
     ----------
     ih : stream handle
         Handle of a stream reader or another task.
+    ih_samples_per_frame : int, optional
+        Number of input samples which should be dealt with in one go.
+        If not given, inferred from ``samples_per_frame``; if that
+        is also not given, taken from the underlying stream.
     shape : tuple, optional
         Overall shape of the stream, with first entry the total number
-        of complete samples, and the remainder the sample shape.  By
-        default, identical to the shape of the underlying stream.
+        of complete samples, and the remainder the sample shape.  If
+        not given, the sample shape is that of the underlying stream,
+        and the number of complete samples is inferred from the number
+        of frames implied by ``ih_samples_per_frame``, combined with
+        ``samples_per_frame``. The latter inference also happens if
+        the first entry is ``-1``. If a shape inconsistent with an
+        integer number of frames is given, the task should be able to
+        deal with a partial frame (which, when the sample rate is reduced,
+        will always contain an integer multiple of the reduction factor).
     sample_rate : `~astropy.units.Quantity`, optional
         With units of a rate.  If not given, taken from the underlying
         stream.
     samples_per_frame : int, optional
-        Number of samples the task should handle in one go.  If given,
-        ``shape`` will be adjusted to make the total number of samples
-        an integer multiple of ``samples_per_frame``.  If not given,
-        the number from the underlying stream.
-    frequency : `~astropy.units.Quantity`, optional
-        Frequencies for each channel.  Should be broadcastable to the
-        sample shape.  Default: taken from the underlying stream, if available.
-    sideband : array, optional
-        Whether frequencies are upper (+1) or lower (-1) sideband.
-        Should be broadcastable to the sample shape.
-        Default: taken from the underlying stream, if available.
-    polarization : array or (nested) list of char, optional
-        Polarization labels.  Should broadcast to the sample shape,
-        i.e., the labels are in the correct axis.  For instance,
-        ``['X', 'Y']``, or ``[['L'], ['R']]``.  Default: taken from the
-        underlying stream, if available.
-    dtype : `~numpy.dtype`, optional
-        Output dtype.  If not given, the dtype of the underlying stream.
+        Number of samples the task produces per frame.  If not given,
+        inferred from the input number of samples using the ratio of
+        the ``sample_rate`` passed in and that of the underlying stream.
+    **kwargs
+        Possible further arguments; see `~baseband_tasks.base.BaseTaskBase`.
     """
 
-    def __init__(self, ih, *,
+    def __init__(self, ih, *, ih_samples_per_frame=None,
                  shape=None, sample_rate=None, samples_per_frame=None,
-                 frequency=None, sideband=None, polarization=None,
-                 dtype=None):
+                 **kwargs):
+
         if sample_rate is None:
             sample_rate = ih.sample_rate
             sample_rate_ratio = 1.
         else:
             sample_rate_ratio = (ih.sample_rate / sample_rate).to(1).value
         if samples_per_frame is None:
-            (samples_per_frame, r) = divmod(ih.samples_per_frame
-                                            / sample_rate_ratio, 1.)
-            assert r == 0, "inferred samples per frame must be integer"
+            if ih_samples_per_frame is None:
+                ih_samples_per_frame = ih.samples_per_frame
+            samples_per_frame = ih_samples_per_frame / sample_rate_ratio
+            assert samples_per_frame % 1 == 0, (
+                "inferred samples per frame must be integer")
             samples_per_frame = int(samples_per_frame)
-            self._raw_samples_per_frame = ih.samples_per_frame
-        else:
-            (raw_samples_per_frame, r) = divmod(samples_per_frame
-                                                * sample_rate_ratio, 1.)
-            assert r == 0, "inferred raw samples per frame must be integer"
-            self._raw_samples_per_frame = int(raw_samples_per_frame)
 
-        nraw_frames = ih.shape[0] // self._raw_samples_per_frame
-        if shape is None:
-            shape = (nraw_frames * samples_per_frame,) + ih.shape[1:]
-        else:
-            assert shape[0] <= nraw_frames * samples_per_frame, \
-                "passed in shape[0] too large"
+        elif ih_samples_per_frame is None:
+            ih_samples_per_frame = samples_per_frame * sample_rate_ratio
+            assert ih_samples_per_frame % 1 == 0, (
+                "inferred input samples per frame must be integer")
+            ih_samples_per_frame = int(ih_samples_per_frame)
 
-        super().__init__(ih=ih, shape=shape,
-                         sample_rate=sample_rate,
+        assert ih_samples_per_frame <= ih.shape[0], (
+            "time per frame larger than total time in stream")
+
+        if shape is None or shape[0] == -1:
+            ns = ((ih.shape[0] // ih_samples_per_frame)
+                  * samples_per_frame)
+            shape = (ns,) + (ih.shape[1:] if shape is None else shape[1:])
+
+        super().__init__(ih=ih, ih_samples_per_frame=ih_samples_per_frame,
+                         shape=shape, sample_rate=sample_rate,
                          samples_per_frame=samples_per_frame,
-                         frequency=frequency, sideband=sideband,
-                         polarization=polarization, dtype=dtype)
+                         **kwargs)
+        alignment = max(1, int(sample_rate_ratio))
+        self._ih_stop = (self.ih.shape[0] // alignment) * alignment
+
+    def _seek_frame(self, frame_index):
+        return self.ih.seek(frame_index * self._ih_samples_per_frame)
 
     def _read_frame(self, frame_index):
         # Read data from underlying filehandle.
-        self.ih.seek(frame_index * self._raw_samples_per_frame)
-        data = self.ih.read(self._raw_samples_per_frame)
-        # Apply function to the data.  Note that the read() function
+        start = self._seek_frame(frame_index)
+        stop = min(start + self._ih_samples_per_frame, self._ih_stop)
+        data = self.ih.read(stop-start)
+        # Apply function to the data.  Note that the _get_frame() function
         # in base ensures that our offset pointer is correct.
         return self.task(data)
 
 
+class PaddedTaskBase(TaskBase):
+    """Base for tasks which need more points than they produce.
+
+    Like `~baseband_tasks.base.TaskBase`, subclasses should define:
+
+      ``task(self, data)`` : return processed data from one frame.
+
+    Where ``data`` will contain extra padding.  The ``task`` method has to
+    ensure the right selection is returned, and can use the ``_pad_start``
+    and ``_pad_end`` attributes for this purpose.
+
+    Parameters
+    ----------
+    ih : stream handle
+        Handle of a stream reader or another task.
+    pad_start, pad_end : int
+        Padding to apply at the start and end.  Default: 0.
+    samples_per_frame : int, optional
+        Number of output samples which should be produced in each frame.
+        The number of input samples will be larger by the padding.
+        If not given, the minimum power of 2 needed to get at least 75%
+        efficiency.
+    **kwargs
+        Possible further arguments; see `~baseband_tasks.base.BaseTaskBase`.
+
+    """
+
+    def __init__(self, ih, pad_start=0, pad_end=0, *,
+                 samples_per_frame=None, **kwargs):
+        self._pad_start = operator.index(pad_start)
+        self._pad_end = operator.index(pad_end)
+        if self._pad_start < 0 or self._pad_end < 0:
+            raise ValueError("padding values must be 0 or positive.")
+
+        pad = self._pad_start + self._pad_end
+        if samples_per_frame is None:
+            # Calculate the number of samples that ensures >75% efficiency:
+            # use 4 times power of two just above pad.
+            ih_samples_per_frame = ih.samples_per_frame
+            if pad > 0:
+                ih_samples_per_frame = max(ih_samples_per_frame, 2 ** (
+                    int((np.ceil(np.log2(pad)))) + 2))
+            samples_per_frame = ih_samples_per_frame - pad
+        else:
+            ih_samples_per_frame = samples_per_frame + pad
+
+        if pad > samples_per_frame:
+            warnings.warn("task will be inefficient; for {} samples "
+                          "per frame, more ({}) will be added for padding."
+                          .format(samples_per_frame, pad))
+
+        n_sample = (((ih.shape[0] - pad) // samples_per_frame)
+                    * samples_per_frame)
+        shape = (n_sample,) + ih.sample_shape
+        start_time = ih.start_time + self._pad_start / ih.sample_rate
+        super().__init__(ih, ih_samples_per_frame=ih_samples_per_frame,
+                         shape=shape, samples_per_frame=samples_per_frame,
+                         start_time=start_time, **kwargs)
+
+    def _seek_frame(self, frame_index):
+        return self.ih.seek(frame_index * self.samples_per_frame)
+
+
 class Task(TaskBase):
     """Apply a user-supplied callable to a stream.
 
@@ -660,66 +809,68 @@ class Task(TaskBase):
 
         super().__init__(ih, **kwargs)
 
+    def _repr_item(self, key, default, value=None):
+        if key == 'task' and isinstance(self.task, types.MethodType):
+            value = self.task.__func__
+        return super()._repr_item(key, default=default, value=value)
 
-class PaddedTaskBase(BaseTaskBase):
-    """Base for tasks which need more points than they produce.
 
-    Like `~baseband_tasks.base.TaskBase`, subclasses should define:
+class SetAttribute(TaskBase):
+    """Wrapper for streams that allows one to set or change attributes.
 
-      ``task(self, data)`` : return processed data from one frame.
+    Can be used to add ``frequency``, ``sideband`` and ``polarization``
+    attributes, which most baseband readers do not provide, checking that
+    the values broadcast properly to the sample shape.
 
-    Where ``data`` will contain extra padding.  The ``task`` method has to
-    ensure the right selection is returned, and can use the ``_pad_start``
-    and ``_pad_end`` attributes for this purpose.
+    Can also be used to apply a clock correction by changing ``start_time``.
+
+    The ``sample_rate`` can also be set, but no check is done to ensure it
+    remains consistent with the ``frequency``.
+
+    One can also change the ``shape``, ``samples_per_frames``, or ``dtype``
+    but that will be slightly slower, as the class can then not read directly
+    from the underlying stream.
+
+    By default, all parameters are taken from the underlying stream.
 
     Parameters
     ----------
     ih : stream handle
         Handle of a stream reader or another task.
-    pad_start, pad_end : int
-        Padding to apply at the start and end.  Default: 0.
-    samples_per_frame : int, optional
-        Number of samples which should be dealt with in one go. The number of
-        output samples per frame will be smaller by the amount of padding.
-        If not given, the minimum power of 2 needed to get at least 75%
-        efficiency.
+    start_time : `~astropy.time.Time`, optional
+        Start time of the stream.
+    sample_rate : `~astropy.units.Quantity`, optional
+        With units of a rate.
+    frequency : `~astropy.units.Quantity`, optional
+        Frequencies for each channel.  Should be broadcastable to the
+        sample shape.
+    sideband : array, optional
+        Whether frequencies are upper (+1) or lower (-1) sideband.
+        Should be broadcastable to the sample shape.
+    polarization : array or (nested) list of char, optional
+        Polarization labels.  Should broadcast to the sample shape,
+        i.e., the labels are in the correct axis.  For instance,
+        ``['X', 'Y']``, or ``[['L'], ['R']]``.
     **kwargs
-        Possible further arguments; see `~baseband_tasks.base.BaseTaskBase`.
-
+        Any other parameters.  See `~baseband_tasks.base.TaskBase`.
     """
 
-    def __init__(self, ih, pad_start=0, pad_end=0, *,
-                 samples_per_frame=None, **kwargs):
-        self._pad_start = operator.index(pad_start)
-        self._pad_end = operator.index(pad_end)
-        if self._pad_start < 0 or self._pad_end < 0:
-            raise ValueError("padding values must be 0 or positive.")
+    def __init__(self, ih, *, start_time=None, sample_rate=None,
+                 frequency=None, sideband=None, polarization=None,
+                 **kwargs):
+        super().__init__(ih, start_time=start_time, sample_rate=sample_rate,
+                         frequency=frequency, sideband=sideband,
+                         polarization=polarization, **kwargs)
+        if not kwargs:
+            # No overrides of anything related to data, so can use read of
+            # underlying file directly.
+            self.read = self.simple_read
 
-        pad = self._pad_start + self._pad_end
-        if pad > 0:
-            if samples_per_frame is None:
-                # Calculate the number of samples that ensures >75% efficiency:
-                # use 4 times power of two just above pad.
-                samples_per_frame = 2 ** (int((np.ceil(np.log2(pad)))) + 2)
-            elif pad >= samples_per_frame:
-                raise ValueError("need more than {} samples per frame to have "
-                                 "enough padding.".format(pad))
-            elif pad > samples_per_frame / 2.:
-                warnings.warn("task will be inefficient since of {} samples "
-                              "per frame, {} will be lost due to padding."
-                              .format(samples_per_frame, pad))
+    def simple_read(self, *args, **kwargs):
+        """Read data from the underlying stream at the current offset."""
+        # Used as ``read`` if the data aspects are not changed. See above.
+        self.ih.seek(self.offset)
+        return self.ih.read(*args, **kwargs)
 
-            # Subtract padding since that's actually produced per frame.
-            samples_per_frame -= pad
-
-        shape = (ih.shape[0] - pad,) + ih.sample_shape
-        super().__init__(ih, shape=shape, samples_per_frame=samples_per_frame,
-                         **kwargs)
-        self._padded_samples_per_frame = self.samples_per_frame + pad
-        self._start_time += self._pad_start / ih.sample_rate
-
-    def _read_frame(self, frame_index):
-        # Read data from underlying filehandle.
-        self.ih.seek(frame_index * self.samples_per_frame)
-        data = self.ih.read(self._padded_samples_per_frame)
-        return self.task(data)
+    def task(self, data):
+        return data
