@@ -18,14 +18,14 @@ class ReshapeTime(TaskBase):
     `ReshapeTime` simply reshapes blocks of baseband data into frames.
     """
 
-    def __init__(self, ih, n, samples_per_frame=1, **kwargs):
+    def __init__(self, ih, n, samples_per_frame=1, fix_shape=False,
+                 **kwargs):
 
-        n = operator.index(n)
+        self._n = n = operator.index(n)
         samples_per_frame = operator.index(samples_per_frame)
         sample_rate = ih.sample_rate / n
-
-        nsample = samples_per_frame * (ih.shape[0] // n // samples_per_frame)
-        super().__init__(ih, shape=(nsample, n) + ih.shape[1:],
+        sh0 = ih.shape[0] // n if fix_shape else -1
+        super().__init__(ih, shape=(sh0, n) + ih.shape[1:],
                          sample_rate=sample_rate,
                          samples_per_frame=samples_per_frame, **kwargs)
 
@@ -56,10 +56,10 @@ class SquareHat(PaddedTaskBase):
     `SquareHat` simply convolves with a set of 1s of given length.
     """
 
-    def __init__(self, ih, n, offset=0, samples_per_frame=None):
+    def __init__(self, ih, n, offset=0, **kwargs):
         self._n = n
         super().__init__(ih, pad_start=n-1-offset, pad_end=offset,
-                         samples_per_frame=samples_per_frame)
+                         **kwargs)
 
     def task(self, data):
         size = data.shape[0]
@@ -124,6 +124,28 @@ class TestSetAttribute(UseVDIFSample):
         with pytest.raises(ValueError):
             SetAttribute(self.fh, sideband=sideband)
 
+    def test_samples_per_frame(self):
+        expected = self.fh.read()
+        sa = SetAttribute(self.fh, samples_per_frame=11111)
+        assert sa.shape == (33333, 8)
+        data = sa.read()
+        assert np.all(data == expected[:33333])
+
+        sa2 = SetAttribute(self.fh, samples_per_frame=11111,
+                           shape=self.fh.shape)
+        assert sa2.shape == self.fh.shape
+        data2 = sa2.read()
+        assert np.all(data2 == expected)
+
+    def test_dtype(self):
+        expected = self.fh.read()
+        sa = SetAttribute(self.fh, dtype='f2')
+        assert isinstance(sa.dtype, np.dtype)
+        assert sa.dtype == 'f2'
+        data = sa.read()
+        assert data.dtype == 'f2'
+        assert np.all(data == expected.astype('f2'))
+
 
 class TestTaskBase(UseVDIFSample):
     def test_basetaskbase(self):
@@ -136,12 +158,14 @@ class TestTaskBase(UseVDIFSample):
         assert mh.ndim == fh.ndim
         assert mh.tell() == 0
         assert mh.tell(unit='time') == mh.time == mh.start_time
-        assert mh.stop_time == fh.stop_time
+        # stop_time is calculated via frame_rate in baseband and via
+        # sample_rate here, so can be slightly different.
+        assert abs(mh.stop_time - fh.stop_time) <= 2**-51 * u.day
 
         expected = fh.read() * 2.
         data = mh.read()
         assert np.all(data == expected)
-        assert mh.time == fh.stop_time
+        assert mh.time == mh.stop_time
         # Check closing.
         mh.close()
         with pytest.raises(ValueError):
@@ -153,17 +177,24 @@ class TestTaskBase(UseVDIFSample):
     def convert_time_offset(offset, sample_rate):
         return int((offset * sample_rate).to(u.one).round())
 
-    @pytest.mark.parametrize(('n', 'samples_per_frame'),
-                             tuple(itertools.product([256, 337], [1, 7, 16])))
-    def test_taskbase(self, n, samples_per_frame):
+    @pytest.mark.parametrize(('n', 'samples_per_frame', 'fix_shape'),
+                             tuple(itertools.product(
+                                 [256, 337],
+                                 [1, 7, 16],
+                                 [True, False])))
+    def test_taskbase(self, n, samples_per_frame, fix_shape):
         """Test properties and methods of TaskBase, including
-        self-consistency with varying ``n`` and ``samples_per_frame``.
+        self-consistency with varying ``n`` and ``samples_per_frame``,
+        and checking that we can get the full data if we fix the shape.
         """
         fh = self.fh
-        rt = ReshapeTime(fh, n, samples_per_frame=samples_per_frame)
+        rt = ReshapeTime(fh, n, samples_per_frame=samples_per_frame,
+                         fix_shape=fix_shape)
         # Check sample pointer.
         assert rt.sample_rate == fh.sample_rate / n
-        nsample = samples_per_frame * (fh.shape[0] // n // samples_per_frame)
+        nsample = fh.shape[0] // n
+        if not fix_shape:
+            nsample = (nsample // samples_per_frame) * samples_per_frame
         assert rt.shape == (nsample, n) + fh.sample_shape
         assert rt.size == np.prod(rt.shape)
         assert rt.ndim == fh.ndim + 1
@@ -254,6 +285,25 @@ class TestTaskBase(UseVDIFSample):
             assert np.all(rt.sideband == sideband_in)
             assert np.all(rt.frequency == frequency_in)
 
+    def test_repr_str(self):
+        fh = self.fh
+        # Add frequency and sideband information by hand, broadcasting it.
+        # (Note: sideband is incorrect; just for testing purposes)
+        frequency_in = 311.25 * u.MHz + (np.arange(8.) // 2) * 16. * u.MHz
+        sideband_in = np.tile([-1, +1], 4)
+        rt = ReshapeTime(fh, 256, frequency=frequency_in,
+                         sideband=sideband_in)
+        r = repr(rt)
+        assert r.startswith('ReshapeTime(ih,')
+        assert 'n=256' in r
+        assert 'frequency=' in r
+        assert 'sideband=' in r
+        assert 'ih: <VDIF' in r
+
+        s = str(rt)
+        # String only keeps named arguments.
+        assert s == 'ReshapeTime(ih, n=256)'
+
     def test_taskbase_exceptions(self):
         """Test exceptions in TaskBase."""
         fh = self.fh
@@ -377,6 +427,26 @@ class TestTasks(UseVDIFSample):
         assert np.all(data1 == ref_data)
         assert ft.closed
 
+    def test_task_function_repr(self):
+        ft = Task(self.fh, zero_channel_4)
+        r = repr(ft)
+        assert r.startswith('Task(ih')
+        assert 'task=' in r
+        assert 'zero_channel_4' in r
+        pre, post = r.split('\nih')
+        assert 'samples_per_frame' not in pre
+        assert 'VDIFStream' in post
+
+    def test_task_method_repr(self):
+        ft = Task(self.fh, zero_every_8th_complex)
+        r = repr(ft)
+        assert r.startswith('Task(ih')
+        assert 'task=' in r
+        assert 'zero_every_8th' in r
+        pre, post = r.split('\nih')
+        assert 'samples_per_frame' not in pre
+        assert 'VDIFStream' in post
+
     def test_invalid(self):
         with pytest.raises(Exception):  # Cannot determine function/method.
             Task(self.fh, np.add)
@@ -401,16 +471,20 @@ class TestTasks(UseVDIFSample):
 
 
 class TestPaddedTaskBase(UseVDIFSample):
-    def test_basics(self):
+    @pytest.mark.parametrize('n', [3, 1])
+    def test_basics(self, n):
         fh = self.fh
-        sh = SquareHat(fh, 3)
-        expected_size = ((fh.shape[0] - 2) // 4) * 4
+        sh = SquareHat(fh, n)
+        expected_size = ((fh.shape[0] - n + 1) // 4) * 4
         assert sh.sample_rate == fh.sample_rate
         assert sh.shape == (expected_size,) + fh.shape[1:]
         assert abs(sh.start_time
-                   - fh.start_time - 2. / fh.sample_rate) < 1. * u.ns
-        raw = fh.read(12)
-        expected = raw[:-2] + raw[1:-1] + raw[2:]
+                   - fh.start_time - (n-1) / fh.sample_rate) < 1. * u.ns
+        if n == 3:
+            raw = fh.read(12)
+            expected = raw[:-2] + raw[1:-1] + raw[2:]
+        elif n == 1:
+            expected = fh.read(10)
         data = sh.read(10)
         assert np.all(data == expected)
         sh.close()
@@ -421,7 +495,64 @@ class TestPaddedTaskBase(UseVDIFSample):
             SquareHat(self.fh, -1)
         with pytest.raises(ValueError):
             SquareHat(self.fh, 10, offset=-1)
-        with pytest.raises(ValueError):
-            SquareHat(self.fh, 10, samples_per_frame=9)
         with pytest.warns(UserWarning, match='inefficient'):
-            SquareHat(self.fh, 10, samples_per_frame=11)
+            SquareHat(self.fh, 10, samples_per_frame=8)
+
+
+class TestSlicingAndArray(UseVDIFSample):
+    @pytest.mark.parametrize('cls,kwargs', [
+        (Multiply, dict(factor=2)),
+        (ReshapeTime, dict(n=5, samples_per_frame=2, fix_shape=True)),
+        (ReshapeTime, dict(n=5, samples_per_frame=2, fix_shape=False)),
+        (SquareHat, dict(n=3)),
+        (SetAttribute,
+         dict(frequency=311.25*u.MHz + (np.arange(8.)//2)*16.*u.MHz,
+              sideband=np.tile([-1, +1], 4))),
+    ])
+    @pytest.mark.parametrize('item', [
+        slice(20, 40),
+        slice(500),
+        slice(-20, None),
+        slice(None),
+        slice(-20, -10)])
+    def test_sample_slice(self, item, cls, kwargs):
+        fh = self.fh
+        instance = cls(fh, **kwargs)
+        expected = instance.read()[item]
+        sliced = instance[item]
+
+        start, stop, _ = item.indices(instance.shape[0])
+        assert sliced.tell() == 0
+        assert sliced.time == sliced.start_time
+        assert sliced.sample_rate == instance.sample_rate
+        assert abs(sliced.start_time - (
+            instance.start_time + start/instance.sample_rate)) < 1.*u.ns
+        assert abs(sliced.stop_time - (
+            instance.start_time + stop/instance.sample_rate)) < 1.*u.ns
+        assert sliced.shape == expected.shape
+        if isinstance(instance, SetAttribute):
+            assert np.all(sliced.sideband == np.tile([-1, +1], 4))
+
+        sliced.seek(-5, 2)
+        data2 = sliced.read()
+        assert np.all(data2 == expected[-5:])
+        # Also test interpretation as array.
+        data = np.array(sliced)
+        assert np.all(data == expected)
+
+    def test_asanyarray(self):
+        expected = self.fh.read() * 4
+        sh = Multiply(self.fh, factor=4.)
+        data = np.asanyarray(sh)
+        assert np.all(data == expected)
+
+    def test_no_implicit_array(self):
+        sh = Multiply(self.fh, factor=4.)
+        # Test that we do not just transform to an array
+        # (picking functions unlikely ever to be whitelisted)
+        with pytest.raises(TypeError):
+            np.array(1.) | sh
+        with pytest.raises(TypeError):
+            np.sin(sh)
+        with pytest.raises(TypeError):
+            np.rot90(sh)
